@@ -1058,7 +1058,8 @@ video_texture_new_from_data (CoglContext *ctx,
   tex = cogl_texture_2d_new_from_data (ctx, width, height, format,
                                        rowstride, data, &internal_error);
   if (tex == NULL) {
-    GST_WARNING ("Cannot allocate Cogl texture : %s", internal_error->message);
+    GST_WARNING ("Cannot create Cogl texture : %s", internal_error->message);
+    cogl_error_free (internal_error);
     return NULL;
   }
 
@@ -1565,7 +1566,7 @@ clutter_gst_ayuv_upload (ClutterGstVideoSink *sink,
                          GstBuffer *buffer)
 {
   ClutterGstVideoSinkPrivate *priv = sink->priv;
-  CoglPixelFormat format = COGL_PIXEL_FORMAT_RGBA_8888;
+  CoglPixelFormat format = COGL_PIXEL_FORMAT_RGBA_8888_PRE;
   GstVideoFrame frame;
 
   if (!gst_video_frame_map (&frame, &priv->info, buffer, GST_MAP_READ))
