@@ -3,7 +3,7 @@
  *
  * GStreamer integration library for Clutter.
  *
- * video-player.c - A simple video player with an OSD.
+ * video-player.c - A simple video player with an OSD using ClutterGstVideoActor.
  *
  * Copyright (C) 2007,2008 OpenedHand
  * Copyright (C) 2013 Collabora
@@ -29,6 +29,8 @@
 #include <clutter/clutter.h>
 #include <clutter-gst/clutter-gst.h>
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
+
 #define SEEK_H 14
 #define SEEK_W 440
 
@@ -38,7 +40,8 @@ typedef struct _VideoApp
 {
   ClutterActor *stage;
 
-  ClutterActor *vtexture;
+  ClutterActor *vactor;
+  ClutterGstPlayback *player;
 
   ClutterActor *control;
   ClutterActor *control_bg;
@@ -156,21 +159,21 @@ show_controls (VideoApp *app, gboolean vis)
 void
 toggle_pause_state (VideoApp *app)
 {
-  if (app->vtexture == NULL)
+  if (app->vactor == NULL)
     return;
 
   if (app->paused)
     {
-      clutter_media_set_playing (CLUTTER_MEDIA(app->vtexture),
-                                 TRUE);
+      clutter_gst_player_set_playing (CLUTTER_GST_PLAYER (app->player),
+                                      TRUE);
       app->paused = FALSE;
       clutter_actor_hide (app->control_play);
       clutter_actor_show (app->control_pause);
     }
   else
     {
-      clutter_media_set_playing (CLUTTER_MEDIA(app->vtexture),
-                                 FALSE);
+      clutter_gst_player_set_playing (CLUTTER_GST_PLAYER (app->player),
+                                      FALSE);
       app->paused = TRUE;
       clutter_actor_hide (app->control_pause);
       clutter_actor_show (app->control_play);
@@ -181,8 +184,8 @@ static void
 reset_animation (ClutterTimeline  *animation,
                  VideoApp         *app)
 {
-  if (app->vtexture)
-    clutter_actor_set_rotation_angle (app->vtexture, CLUTTER_Y_AXIS, 0.0);
+  if (app->vactor)
+    clutter_actor_set_rotation_angle (app->vactor, CLUTTER_Y_AXIS, 0.0);
 }
 
 static gboolean
@@ -230,8 +233,7 @@ input_cb (ClutterStage *stage,
 
               progress = (gdouble) dist / SEEK_W;
 
-              clutter_media_set_progress (CLUTTER_MEDIA (app->vtexture),
-                                          progress);
+              clutter_gst_playback_set_progress (app->player, progress);
             }
         }
       handled = TRUE;
@@ -244,10 +246,10 @@ input_cb (ClutterStage *stage,
         switch (clutter_event_get_key_symbol (event))
           {
           case CLUTTER_d:
-            if (app->vtexture)
+            if (app->vactor)
               {
-                clutter_actor_remove_child (app->stage, app->vtexture);
-                app->vtexture = NULL;
+                clutter_actor_remove_child (app->stage, app->vactor);
+                app->vactor = NULL;
               }
             if (app->control)
               {
@@ -261,12 +263,12 @@ input_cb (ClutterStage *stage,
             break;
 
           case CLUTTER_e:
-            if (app->vtexture == NULL)
+            if (app->vactor == NULL)
               break;
 
-            clutter_actor_set_pivot_point (app->vtexture, 0.5, 0);
+            clutter_actor_set_pivot_point (app->vactor, 0.5, 0);
 
-            animation = _actor_animate (app->vtexture, CLUTTER_LINEAR, 500,
+            animation = _actor_animate (app->vactor, CLUTTER_LINEAR, 500,
                                         "rotation-angle-y", 360.0, NULL);
 
             g_signal_connect_after (animation, "completed",
@@ -299,45 +301,6 @@ input_cb (ClutterStage *stage,
   return handled;
 }
 
-static void
-size_change (ClutterTexture *texture,
-             gint            base_width,
-             gint            base_height,
-             VideoApp       *app)
-{
-  ClutterActor *stage = app->stage;
-  gfloat new_x, new_y, new_width, new_height;
-  gfloat stage_width, stage_height;
-  gfloat frame_width, frame_height;
-
-  clutter_actor_get_size (stage, &stage_width, &stage_height);
-
-  /* base_width and base_height are the actual dimensions of the buffers before
-   * taking the pixel aspect ratio into account. We need to get the actual
-   * size of the texture to display */
-  clutter_actor_get_size (CLUTTER_ACTOR (texture), &frame_width, &frame_height);
-
-  new_height = (frame_height * stage_width) / frame_width;
-  if (new_height <= stage_height)
-    {
-      new_width = stage_width;
-
-      new_x = 0;
-      new_y = (stage_height - new_height) / 2;
-    }
-  else
-    {
-      new_width  = (frame_width * stage_height) / frame_height;
-      new_height = stage_height;
-
-      new_x = (stage_width - new_width) / 2;
-      new_y = 0;
-    }
-
-  clutter_actor_set_position (CLUTTER_ACTOR (texture), new_x, new_y);
-  clutter_actor_set_size (CLUTTER_ACTOR (texture), new_width, new_height);
-}
-
 static void
 position_controls (VideoApp     *app,
                    ClutterActor *controls)
@@ -368,8 +331,7 @@ tick (GObject      *object,
       GParamSpec   *pspec,
       VideoApp     *app)
 {
-  ClutterMedia *video_texture = CLUTTER_MEDIA (object);
-  gdouble progress = clutter_media_get_progress (video_texture);
+  gdouble progress = clutter_gst_playback_get_progress (app->player);
 
   clutter_actor_set_size (app->control_seekbar,
                           progress * SEEK_W,
@@ -377,13 +339,13 @@ tick (GObject      *object,
 }
 
 static void
-on_video_texture_eos (ClutterMedia *media,
-                      VideoApp     *app)
+on_video_actor_eos (ClutterGstPlayer *player,
+                    VideoApp         *app)
 {
   if (opt_loop)
     {
-      clutter_media_set_progress (media, 0.0);
-      clutter_media_set_playing (media, TRUE);
+      clutter_gst_playback_set_progress (CLUTTER_GST_PLAYBACK (player), 0.0);
+      clutter_gst_player_set_playing (player, TRUE);
     }
 }
 
@@ -396,9 +358,73 @@ _new_rectangle_with_color (ClutterColor *color)
   return actor;
 }
 
+static ClutterActor *
+control_actor_new_from_image (const gchar *image_filename)
+{
+  ClutterActor *actor;
+  ClutterContent *image;
+  GdkPixbuf *pixbuf;
+
+  actor = clutter_actor_new ();
+
+  image = clutter_image_new ();
+  pixbuf = gdk_pixbuf_new_from_file (image_filename, NULL);
+  clutter_image_set_data (CLUTTER_IMAGE (image),
+                          gdk_pixbuf_get_pixels (pixbuf),
+                          gdk_pixbuf_get_has_alpha (pixbuf)
+                            ? COGL_PIXEL_FORMAT_RGBA_8888
+                            : COGL_PIXEL_FORMAT_RGB_888,
+                          gdk_pixbuf_get_width (pixbuf),
+                          gdk_pixbuf_get_height (pixbuf),
+                          gdk_pixbuf_get_rowstride (pixbuf),
+                          NULL);
+  clutter_actor_set_size (actor,
+                          gdk_pixbuf_get_width (pixbuf),
+                          gdk_pixbuf_get_height (pixbuf));
+  g_object_unref (pixbuf);
+
+  clutter_actor_set_content (actor, image);
+  g_object_unref (image);
+
+  return actor;
+}
+
+static void
+on_video_actor_notify_buffer_fill (GObject    *selector,
+                                   GParamSpec *pspec,
+                                   VideoApp   *app)
+{
+  gdouble buffer_fill;
+
+  g_object_get (app->player, "buffer-fill", &buffer_fill, NULL);
+  g_print ("Buffering - percentage=%d%%\n", (int) (buffer_fill * 100));
+}
+
+/* check whether a given uri is a local file.
+ * Note that this method won't check if the uri exists,
+ * just if the uri is a valid uri for a local file */
+static gboolean
+is_local_file (const gchar *uri)
+{
+  gboolean ret = FALSE;
+  gchar *scheme;
+
+  scheme = g_uri_parse_scheme (uri);
+
+  /* uris can be considered local if using file:// or
+   * using a relative path/no scheme */
+  if (!scheme || !g_ascii_strcasecmp (scheme, "file"))
+    ret = TRUE;
+
+  g_free (scheme);
+  return ret;
+}
+
 int
 main (int argc, char *argv[])
 {
+  const gchar         *uri;
+  gchar               *scheme;
   VideoApp            *app = NULL;
   GstElement          *pipe;
   GstElement          *playsink;
@@ -429,10 +455,12 @@ main (int argc, char *argv[])
 
   if (argc < 2)
     {
-      g_print ("Usage: %s [OPTIONS] <video file>\n", argv[0]);
+      g_print ("Usage: %s [OPTIONS] <video uri>\n", argv[0]);
       return EXIT_FAILURE;
     }
 
+  uri = argv[1];
+
   stage = clutter_stage_new ();
   clutter_actor_set_background_color (stage, &stage_color);
   clutter_actor_set_size (stage, 768, 576);
@@ -442,25 +470,46 @@ main (int argc, char *argv[])
 
   app = g_new0(VideoApp, 1);
   app->stage = stage;
-  app->vtexture = clutter_gst_video_texture_new ();
+  app->player = clutter_gst_playback_new ();
 
-  if (app->vtexture == NULL)
-    g_error("failed to create vtexture");
+  app->vactor = g_object_new (CLUTTER_TYPE_ACTOR,
+                              "width", clutter_actor_get_width (stage),
+                              "height", clutter_actor_get_height (stage),
+                              "content", g_object_new (CLUTTER_GST_TYPE_ASPECTRATIO,
+                                                       "player", app->player,
+                                                       NULL),
+                              NULL);
+
+  if (app->vactor == NULL)
+    g_error("failed to create vactor");
 
   /* By default ClutterGst seeks to the nearest key frame (faster). However
    * it has the weird effect that when you click on the progress bar, the fill
    * goes to the key frame position that can be quite far from where you
    * clicked. Using the ACCURATE flag tells playbin2 to seek to the actual
    * frame */
-  clutter_gst_video_texture_set_seek_flags (
-                                    CLUTTER_GST_VIDEO_TEXTURE (app->vtexture),
-                                    CLUTTER_GST_SEEK_FLAG_ACCURATE);
+  clutter_gst_playback_set_seek_flags (app->player,
+                                       CLUTTER_GST_SEEK_FLAG_ACCURATE);
 
-  g_signal_connect (app->vtexture,
+  g_signal_connect (app->player,
                     "eos",
-                    G_CALLBACK (on_video_texture_eos),
+                    G_CALLBACK (on_video_actor_eos),
                     app);
 
+  if (!is_local_file (uri))
+    {
+      g_print ("Remote media detected, setting up buffering\n");
+
+      clutter_gst_playback_set_buffering_mode (app->player,
+        CLUTTER_GST_BUFFERING_MODE_DOWNLOAD);
+      g_signal_connect (app->player,
+                        "notify::buffer-fill",
+                        G_CALLBACK (on_video_actor_notify_buffer_fill),
+                        app);
+    }
+  else
+      g_print ("Local media detected\n");
+
   g_signal_connect (stage,
                     "allocation-changed",
                     G_CALLBACK (on_stage_allocation_changed),
@@ -471,16 +520,22 @@ main (int argc, char *argv[])
                     G_CALLBACK (clutter_main_quit),
                     NULL);
 
-  /* Handle it ourselves so can scale up for fullscreen better */
-  g_signal_connect_after (CLUTTER_TEXTURE (app->vtexture),
-                          "size-change",
-                          G_CALLBACK (size_change), app);
+  /* Load up out video actor */
+  scheme = g_uri_parse_scheme (uri);
+  if (scheme != NULL)
+    clutter_gst_playback_set_uri (app->player, uri);
+  else
+    clutter_gst_playback_set_filename (app->player, uri);
 
-  /* Load up out video texture */
-  clutter_media_set_filename (CLUTTER_MEDIA (app->vtexture), argv[1]);
+  g_free (scheme);
+
+  if (clutter_gst_playback_is_live_media (app->player))
+    g_print ("Playing live media\n");
+  else
+    g_print ("Playing non-live media\n");
 
   /* Set up things so that a visualisation is played if there's no video */
-  pipe = clutter_gst_video_texture_get_pipeline (CLUTTER_GST_VIDEO_TEXTURE (app->vtexture));
+  pipe = clutter_gst_player_get_pipeline (CLUTTER_GST_PLAYER (app->player));
   if (!pipe)
     g_error ("Unable to get gstreamer pipeline!\n");
 
@@ -513,11 +568,11 @@ main (int argc, char *argv[])
   app->control = clutter_actor_new ();
 
   app->control_bg =
-    clutter_texture_new_from_file ("vid-panel.png", NULL);
+    control_actor_new_from_image ("vid-panel.png");
   app->control_play =
-    clutter_texture_new_from_file ("media-actions-start.png", NULL);
+    control_actor_new_from_image ("media-actions-start.png");
   app->control_pause =
-    clutter_texture_new_from_file ("media-actions-pause.png", NULL);
+    control_actor_new_from_image ("media-actions-pause.png");
 
   g_assert (app->control_bg && app->control_play && app->control_pause);
 
@@ -528,7 +583,7 @@ main (int argc, char *argv[])
 
   app->control_label =
     clutter_text_new_full ("Sans Bold 14",
-                           g_path_get_basename (argv[1]),
+                           g_path_get_basename (uri),
                            &control_color1);
 
   clutter_actor_hide (app->control_play);
@@ -556,7 +611,7 @@ main (int argc, char *argv[])
   clutter_actor_set_position (app->control_label, 82, 29);
 
   /* Add control UI to stage */
-  clutter_actor_add_child (stage, app->vtexture);
+  clutter_actor_add_child (stage, app->vactor);
   clutter_actor_add_child (stage, app->control);
 
   position_controls (app, app->control);
@@ -569,11 +624,11 @@ main (int argc, char *argv[])
   /* Hook up other events */
   g_signal_connect (stage, "event", G_CALLBACK (input_cb), app);
 
-  g_signal_connect (app->vtexture,
+  g_signal_connect (app->player,
                     "notify::progress", G_CALLBACK (tick),
                     app);
 
-  clutter_media_set_playing (CLUTTER_MEDIA (app->vtexture), TRUE);
+  clutter_gst_player_set_playing (CLUTTER_GST_PLAYER (app->player), TRUE);
 
   clutter_actor_show (stage);
 
